@@ -1,16 +1,9 @@
-"""
-Reverse DNS Lookup Engine.
-
-This module allows other programs to load in multipe IPv4 or IPv6 addresses and then
-perform a serial or parallel resolving of all addresses.  Filtering options exist
-and a disk based cache is supported to decrease runtime for subsequent uses.
-"""
-
+import concurrent.futures
 import copy
 import socket
-import concurrent.futures
+
 import yaml
-from netaddr import IPAddress, IPSet, AddrFormatError
+from netaddr import AddrFormatError, IPAddress, IPSet
 
 IPV4_RESERVED_NETWORK_LIST = [
     "0.0.0.0/8",
@@ -50,11 +43,9 @@ IPV6_RESERVED_NETWORK_LIST = [
 ]
 
 
-class rdns_reaper:
+class RdnsReaper:
     """Reverse DNS Lookup Engine."""
 
-    _filter = None
-
     def __init__(self, **kwargs):
         """Initialize class and take in user options.
 
@@ -78,42 +69,42 @@ class rdns_reaper:
 
 
         """
-        self._dns_dict = {}
         self._resolver_ip = None
-
-        """Check for RFC1918 filtering"""
+        self._dns_dict = {}
+        self._concurrent = 5
+        self._options_dict = {
+            "allow_reserved_networks": False,
+            "concurrent": 5,
+            "filemode": None,
+            "filename": None,
+            "filter": None,
+            "filter_mode": None,
+            "limit_to_rfc1918": False,
+        }
+        # Check for RFC1918 filtering
         if "limit_to_rfc1918" in kwargs:
             self.limit_to_rfc1918(kwargs["limit_to_rfc1918"])
         else:
-            self._limit_to_rfc1918 = False
+            self._options_dict["limit_to_rfc1918"] = False
 
-        """Check for custom filtering"""
+        # Check for custom filtering
         if kwargs.get("filter") is not None:
-            if kwargs.get("filtermode") is not None:
-                self.set_filter(kwargs["filter"], mode=kwargs["filtermode"])
-            else:
-                self.set_filter(kwargs["filter"])
+            self.set_filter(kwargs.get("filter"), mode=kwargs.get("filtermode"))
+
+        # Allow reserved network check
+        if kwargs.get("allow_reserved_networks"):
+            self._options_dict["allow_reserved_networks"] = True
         else:
-            self._filter = None
-            self._filter_mode = None
+            self._options_dict["allow_reserved_networks"] = False
 
-        """Allow reserved network check"""
-
-        if kwargs.get("allow_reserved_networks") is True:
-            self._allow_reserved_networks = True
-        else:
-            self._allow_reserved_networks = False
-
-        """Process parallel lookup concurrency"""
-        try:
-            if type(kwargs["concurrent"]) is int:
-                self._concurrent = kwargs["concurrent"]
+        # Process parallel lookup concurrency
+        if kwargs.get("concurrent"):
+            if isinstance(kwargs["concurrent"], int):
+                self._options_dict["concurrent"] = kwargs["concurrent"]
             else:
                 raise TypeError
-        except KeyError:
-            self._concurrent = 5
 
-        """Determine how to mark unresolvable entries"""
+        # Determine how to mark unresolvable entries
         try:
             if isinstance(kwargs["unresolvable"], str):
                 self._unresolvable = kwargs["unresolvable"]
@@ -124,30 +115,35 @@ class rdns_reaper:
 
         try:
             if isinstance(kwargs["filemode"], str):
-                self._filemode = kwargs["filemode"]
+                if kwargs["filemode"] not in ("w", "r"):
+                    raise ValueError
+
+                self._options_dict["filemode"] = kwargs["filemode"]
             else:
                 raise TypeError
         except KeyError:
-            self._filemode = None
+            pass
 
         try:
             if isinstance(kwargs["filename"], str):
-                self._filename = kwargs["filename"]
+                self._options_dict["filename"] = kwargs["filename"]
             else:
                 raise TypeError
         except KeyError:
-            self._filename = None
+            pass
 
-        if (self._filename is not None) and ((self._filemode in "r", "w")):
+        if (self._options_dict["filename"] is not None) and (
+            self._options_dict["filemode"] in ("r", "w")
+        ):
             try:
-                self.loadfile(self._filename)
+                self.loadfile(self._options_dict["filename"])
             except FileNotFoundError:
                 pass
 
     def __add__(self, new):
         """Add two instances together and return a deepcopy."""
         self_copy = copy.deepcopy(self)
-        if type(new) is rdns_reaper:
+        if isinstance(new, RdnsReaper):
             self_copy._dns_dict.update(new._dns_dict)
         elif isinstance(new, str):
             self_copy.add_ip(new)
@@ -160,15 +156,7 @@ class rdns_reaper:
         return self_copy
 
     def __contains__(self, ip_address):
-        """Allow for checking of IP in the instance.
-
-        Args:
-            ip_address (str): A string containing an IP address to check
-
-        Returns:
-            bool: True if found, false if not found
-
-        """
+        """Allow for checking of IP in the instance."""
         if ip_address in self._dns_dict.keys():
             return True
         return False
@@ -178,13 +166,13 @@ class rdns_reaper:
         self.remove_ip(ip_address)
 
     def __enter__(self):
-        """Return self for use in with blocks."""
+        """Return self for use in 'with open()' style blocks."""
         return self
 
     def __exit__(self, exception_type, exception_value, traceback):
         """Save file when class is being destroyed as appropriate."""
-        if (self._filename is not None) and (self._filemode == "w"):
-            self.savefile(self._filename)
+        if (self._options_dict["filename"] is not None) and (self._options_dict["filemode"] == "w"):
+            self.savefile(self._options_dict["filename"])
 
     def __getitem__(self, ip_address):
         """Return IP address from internal dictionary, false if not found."""
@@ -192,13 +180,13 @@ class rdns_reaper:
             IPAddress(ip_address)
             return self._dns_dict[ip_address]
         except AddrFormatError as error_case:
-            raise TypeError from error_case
+            raise AddrFormatError from error_case
         except KeyError:
             return False
 
     def __iadd__(self, new):
         """Add two instances together and return."""
-        if type(new) is rdns_reaper:
+        if isinstance(new, RdnsReaper):
             self._dns_dict.update(new._dns_dict)
         elif isinstance(new, str):
             self.add_ip(new)
@@ -212,38 +200,29 @@ class rdns_reaper:
 
     def __iter__(self):
         """Return an iterator as needed for k, v walking."""
-        return self._reaper_iterator(self)
+        return self._ReaperIterator(self)
 
     def __len__(self):
         """Determine number of addresses in module."""
         return len(self._dns_dict)
 
-    def __setitem__(self, ip, value):
+    def __setitem__(self, ip_address, value):
         """Allow set/reset of hostname for an IP via index."""
-        self.setname(ip, value)
+        self.set_name(ip_address, value)
 
-    def _resolve_function(self, ip):
+    def _resolve_function(self, ip_address):
         try:
-            name = socket.gethostbyaddr(ip)[0]
-            self._dns_dict[ip] = name
+            name = socket.gethostbyaddr(ip_address)[0]
+            self._dns_dict[ip_address] = name
         except socket.herror:
-            self._dns_dict[ip] = self._unresolvable
+            self._dns_dict[ip_address] = self._unresolvable
 
-    def add_ip(self, ip_address, hostname=None):
-        """Add an IP to the list with option hostname, skip if exists.
+    def add(self, *args, **kwargs):
+        """Generalized function for adding."""
+        self.__iadd__(*args, **kwargs)
 
-        Args:
-            ip_address (str): A string containing an IP address in v4 or v6 format
-            hostname (str): Optional string with a hostname
-                Without this argument, the Hostname will be set to None and will be eligible
-                for automatic lookup when the resolver functions are called
-                The hostname string should be a FQDN or similar text that would be returned
-                by a true DNS based reverseloopkup.
-
-        Returns:
-            True if already in dictionary, otherwise returns the key string back on success.
-
-        """
+    def add_ip(self, ip_address: str, hostname: str = None):
+        """Add an IP to the list with option hostname, skip if exists."""
         if ip_address in self._dns_dict.keys():
             return True
 
@@ -257,20 +236,16 @@ class rdns_reaper:
         except AddrFormatError as error_case:
             raise TypeError from error_case
 
-    def add_ip_list(self, ip_list):
-        """Add all new IP's from a list.
-
-        Args:
-            ip_address (list): A list of strings each containing an IP address in v4 or v6 format
-        """
+    def add_ip_list(self, ip_list: list):
+        """Add all new IP's from a list."""
         if isinstance(ip_list, set):
             ip_list = list(ip_list)
         if not isinstance(ip_list, list):
             raise TypeError
-        for ip in ip_list:
-            self.add_ip(ip)
+        for ip_address in ip_list:
+            self.add_ip(ip_address)
 
-    def allow_reserved_networks(self, option):
+    def allow_reserved_networks(self, option: bool):
         """Allow users to enable/disable automatic filtering of reserved networks.
 
         If a user wants to check reserved network IPs (loopbacks, link local, multicast, etc.)
@@ -286,102 +261,78 @@ class rdns_reaper:
         if not isinstance(option, bool):
             raise TypeError
 
-        self._allow_reserved_networks = option
+        self._options_dict["allow_reserved_networks"] = option
 
     def clear_all_hostnames(self):
         """Clear all the hostnames from existing entries."""
         new_ip_dict = {ip: None for ip in self._dns_dict}
         self._dns_dict = new_ip_dict
 
-    def clearname(self, ip):
+    def clearname(self, ip_address: str):
         """Clear a specific IP's hostname.
 
         Args:
-            ip (str): A string containing an IP address
+            ip_address (str): A string containing an IP address
         """
         try:
-            ip = IPAddress(ip)
-            self._dns_dict[ip] = None
-            return True
+            IPAddress(ip_address)
         except AddrFormatError as error_case:
             raise TypeError from error_case
-        except KeyError:
-            return False
 
-    def dict(self):
-        """Return the internal dictionary to the calling function."""
-        return self.dict()
+        if ip_address not in self._dns_dict.keys():
+            raise KeyError("Address not found")
 
-    def get_dict(self):
+        self._dns_dict[ip_address] = None
+
+    def dict(self) -> dict:
         """Return the internal dictionary to the calling function."""
         return self._dns_dict
 
-    def get_filter(self):
-        """Return current filter status.
+    def get_dict(self) -> dict:
+        """Return the internal dictionary to the calling function."""
+        return self._dns_dict
 
-        Returns:
-            tuple - (filter, filter_mode)
-                filter is an IPSet object containing all networks that are part of the filter
-                filter_mode is a string of "block" or "allow" to indicate the filtering mode
-                    as a block list or allow list
-        """
-        try:
-            return (self._filter, self._filter_mode)
-        except AttributeError:
-            return None
+    def get_filter(self) -> tuple:
+        """Return current filter status."""
+        return (self._options_dict["filter"], self._options_dict["filter_mode"])
 
-    def get_options(self):
+    def get_options(self) -> dict:
         """Return info about the various options set by the user."""
-        options_dict = {
-            "allow_reserved_networks": self._allow_reserved_networks,
-            "concurrent": self._concurrent,
-            "limit_to_rfc1918": self._limit_to_rfc1918,
-            "filter": self._filter,
-            "filtermode": self._filter_mode,
-            "filename": self._filename,
-            "filemode": self._filemode,
-        }
-        return options_dict
+        return self._options_dict
 
     def items(self):
-        """Return the IP address and hostnames as k, v pairs in list format."""
-        return dict(self._dns_dict.items())
+        """Return the IP address and hostnames as a dict_items to allow iteration over k, v pairs."""
+        return self._dns_dict.items()
 
-    def keys(self):
+    def keys(self) -> list:
         """Return the IP address as keys in list format."""
         return list(self._dns_dict.keys())
 
-    def limit_to_rfc1918(self, option):
-        """Set the RFC1918 filter.
-
-        Args:
-            option (bool): True limits the resolver to only IPv4 RFC1918 networks
-
-        IPv6 resolving is effecitvely disabled when this option is set to True
-        """
-        if not isinstance(option, bool):
+    def limit_to_rfc1918(self, value: bool):
+        """Set the RFC1918 filter."""
+        if not isinstance(value, bool):
             raise TypeError
-        self._limit_to_rfc1918 = option
+        self._options_dict["limit_to_rfc1918"] = value
 
-    def loadfile(self, filename):
+    def loadfile(self, filename: str):
         """Load saved data in YAML format.
 
         Args
             filename (str): path and filename for the disk based YAML cache file
         """
-        with open(filename) as f_handle:
+        with open(filename, encoding="UTF-8") as f_handle:
             f_data = f_handle.read()
             self._dns_dict = yaml.safe_load(f_data)
 
-    def remove_ip(self, ip):
+    def remove_ip(self, ip_address: str) -> bool:
         """Remove an IP from the list, return false if not found.
 
         Args:
-            ip (str): A string containing an IP Address to remove
+            ip_address (str): A string containing an IP Address to remove
         """
         try:
-            IPAddress(ip)
-            self._dns_dict.pop(ip)
+            IPAddress(ip_address)
+            self._dns_dict.pop(ip_address)
         except AddrFormatError as error_case:
             raise TypeError from error_case
         except KeyError:
@@ -390,29 +341,27 @@ class rdns_reaper:
 
     def _build_resolve_list(self):
         """Build list of IP's to perform resolver on, shared by serial and parallel methods."""
-        if self._limit_to_rfc1918:
-            IPv4_skipped_networks = IPSet(IPV4_RESERVED_NETWORK_LIST)
-        elif self._allow_reserved_networks is False:
-            IPv4_skipped_networks = IPSet(IPV4_RESERVED_NETWORK_LIST)
+        if self._options_dict["limit_to_rfc1918"]:
+            ipv4_skipped_networks = IPSet(IPV4_RESERVED_NETWORK_LIST)
+        elif self._options_dict["allow_reserved_networks"] is False:
+            ipv4_skipped_networks = IPSet(IPV4_RESERVED_NETWORK_LIST)
         else:
-            IPv4_skipped_networks = IPSet()
+            ipv4_skipped_networks = IPSet()
 
-        if self._allow_reserved_networks is False:
-            IPv6_skipped_networks = IPSet(IPV6_RESERVED_NETWORK_LIST)
+        if self._options_dict["allow_reserved_networks"] is False:
+            ipv6_skipped_networks = IPSet(IPV6_RESERVED_NETWORK_LIST)
         else:
-            IPv6_skipped_networks = IPSet()
+            ipv6_skipped_networks = IPSet()
 
-        initial_ip_list = [
-            key for key, value in self._dns_dict.items() if value is None
-        ]
+        initial_ip_list = [key for key, value in self._dns_dict.items() if value is None]
 
         pending_ipset = IPSet(initial_ip_list)
 
-        if self._filter_mode == "block":
-            result_ipset = pending_ipset - self._filter
+        if self._options_dict["filter_mode"] == "block":
+            result_ipset = pending_ipset - self._options_dict["filter"]
             initial_pending_ips = [str(x) for x in result_ipset]
-        elif self._filter_mode == "allow":
-            result_ipset = pending_ipset & self._filter
+        elif self._options_dict["filter_mode"] == "allow":
+            result_ipset = pending_ipset & self._options_dict["filter"]
             initial_pending_ips = [str(x) for x in result_ipset]
         else:
             initial_pending_ips = [str(x) for x in pending_ipset]
@@ -422,10 +371,10 @@ class rdns_reaper:
         for key in initial_pending_ips:
             address = IPAddress(key)
 
-            if self._limit_to_rfc1918 is False:
-                if (address.version == 4) and (address not in IPv4_skipped_networks):
+            if self._options_dict["limit_to_rfc1918"] is False:
+                if (address.version == 4) and (address not in ipv4_skipped_networks):
                     pending_ips.append(key)
-                elif (address.version == 6) and (address not in IPv6_skipped_networks):
+                elif (address.version == 6) and (address not in ipv6_skipped_networks):
                     pending_ips.append(key)
             else:
                 if address in IPSet(IPV4_RFC1918_NETWORK_LIST):
@@ -437,7 +386,7 @@ class rdns_reaper:
         """Resolve all unknown IPs in parallel."""
         pending_ips = self._build_resolve_list()
         with concurrent.futures.ThreadPoolExecutor(
-            max_workers=self._concurrent
+            max_workers=self._options_dict["concurrent"]
         ) as executor:
             executor.map(self._resolve_function, set(pending_ips))
 
@@ -448,22 +397,20 @@ class rdns_reaper:
         for address in pending_ips:
             self._resolve_function(address)
 
-    def savefile(self, filename=None):
-        """Save internal dictionary to YAML file.
+    def savefile(self, filename: str = None):
+        """Save internal dictionary to YAML file."""
+        if (
+            filename is None
+            and self._options_dict["filename"] is not None
+            and self._options_dict["filemode"] == "w"
+        ):
+            filename = self._options_dict["filename"]
 
-        Args:
-            filename (str): Optional path and filename for disk based YAML cache file.
-                If not specified, the filename set at instance creation is used
-                If neither are specified, the operation will fail
-        """
-        if filename is None and self._filename is not None and self._filemode == "w":
-            filename = self._filename
-
-        with open(filename, "w") as f_handle:
+        with open(filename, "w", encoding="UTF-8") as f_handle:
             f_handle.write("---\n")
             f_handle.write(yaml.dump(self._dns_dict))
 
-    def setname(self, ip_address, hostname):
+    def set_name(self, ip_address: str, hostname: str):
         """Force the hostname of an IP.
 
         Args:
@@ -472,115 +419,97 @@ class rdns_reaper:
                 Can be None to reset record to allow for subsequent lookup
         """
         try:
-            ip_address = IPAddress(ip_address)
-            self._dns_dict[ip_address] = hostname
-            return hostname
+            IPAddress(ip_address)
         except AddrFormatError as error_case:
-            raise TypeError from error_case
-        except KeyError:
-            return False
+            raise TypeError(f"IP Error: {error_case}") from error_case
 
-    def set_resolver(self, resolver_ip):
-        """Set the desired resolve IP - NOT WORKING."""
-        try:
-            IPAddress(resolver_ip)
-            self._resolver_ip = resolver_ip
-        except AddrFormatError as error_case:
-            raise TypeError from error_case
+        if ip_address not in self._dns_dict.keys():
+            raise KeyError("Address does not exist")
+        self._dns_dict[ip_address] = hostname
+        # except KeyError as error_case:
+        # raise KeyError("Address does not exist")
 
-    def set_filter(self, filter_input, **kwargs):
-        """Set customer filter options.
-
-        Args:
-            filter_input (str, list of strings, IPSet) - custom filter data
-                Data can be a single string containing an IP network, a list of strings
-                containing IP networks, or an IPSet containing networks
-
-            mode=("block" | "allow", optional) - named argument for a block list or allow list
-                a block list will allow anything not specified in the filter data
-                an allow list permits only what is specified in the filter data
-                the default option is a block list if no mode is specified
-        """
+    def set_filter(self, filter_data: str, **kwargs):
+        """Define a custom filter."""
         if kwargs.get("mode") is None:
-            self._filter_mode = "block"
-        elif kwargs.get("mode").lower() == "allow":
-            self._filter_mode = "allow"
-        elif kwargs.get("mode").lower() == "block":
-            self._filter_mode = "block"
+            self._options_dict["filter_mode"] = "block"
+        elif kwargs.get("mode").lower() in ("allow", "block"):
+            self._options_dict["filter_mode"] = kwargs.get("mode").lower()
         else:
             raise ValueError
 
-        if isinstance(filter_input, str):
-            filter_input = IPSet([filter_input])
-        elif isinstance(filter_input, list):
-            filter_input = IPSet(filter_input)
+        if isinstance(filter_data, str):
+            filter_data = IPSet([filter_data])
+        elif isinstance(filter_data, list):
+            filter_data = IPSet(filter_data)
 
-        if not isinstance(filter_input, IPSet):
+        if not isinstance(filter_data, IPSet):
             raise TypeError
 
-        self._filter = filter_input
+        self._options_dict["filter"] = filter_data
 
-    def values(self):
+    def values(self) -> list:
         """Return the hostnames as values in list format."""
         return list(self._dns_dict.values())
 
     @staticmethod
-    def _isrfc1918(address_txt):
+    def _isrfc1918(address_txt) -> bool:
         """Determine if an address is RFC1918 private or not."""
         address = IPAddress(address_txt)
         rfc1918_networks = ["10.0.0.0/8", "172.16.0.0/12", "192.168.0.0/16"]
-        rfc1918_IPSet = IPSet(rfc1918_networks)
+        rfc1918_ipset = IPSet(rfc1918_networks)
 
-        if address in rfc1918_IPSet:
+        if address in rfc1918_ipset:
             return True
         return False
 
     @staticmethod
-    def _isreservedIPv4(address_txt):
+    def _isreservedaddress(address_txt) -> bool:
         """Determine if an address is reserved (loopbacks, documentation, etc) or not."""
         address = IPAddress(address_txt)
 
         if address.version == 4:
-            reserved_network_IPSet = IPSet(IPV4_RESERVED_NETWORK_LIST)
-            if address_txt in reserved_network_IPSet:
-                return True
-            return False
-
+            return RdnsReaper._isreservedipv4(address_txt)
         if address.version == 6:
-            reserved_network_IPSet = IPSet(IPV6_RESERVED_NETWORK_LIST)
-            if address_txt in reserved_network_IPSet:
+            return RdnsReaper._isreservedipv6(address_txt)
+
+    @staticmethod
+    def _isreservedipv4(address_txt) -> bool:
+        """Determine if an address is reserved (loopbacks, documentation, etc) or not."""
+        address = IPAddress(address_txt)
+
+        if address.version == 4:
+            reserved_network_ipset = IPSet(IPV4_RESERVED_NETWORK_LIST)
+            if address_txt in reserved_network_ipset:
                 return True
             return False
 
         raise ValueError
 
     @staticmethod
-    def _isreservedIPv6(address_txt):
+    def _isreservedipv6(address_txt) -> bool:
         """Determine if an address is reserved (loopbacks, documentation, etc) or not."""
         address = IPAddress(address_txt)
 
         if address.version == 6:
-            reserved_network_IPSet = IPSet(IPV6_RESERVED_NETWORK_LIST)
-            if address_txt in reserved_network_IPSet:
+            reserved_network_ipset = IPSet(IPV6_RESERVED_NETWORK_LIST)
+            if address_txt in reserved_network_ipset:
                 return True
             return False
 
         raise ValueError
 
-    class _reaper_iterator:
+    class _ReaperIterator:
         def __init__(self, parentclass):
             self.__parentclass = parentclass
             self.__counter = 0
             self.__parent_len = len(parentclass)
-            self.__parent_keys = list(parentclass._dns_dict.keys())
-
-        def __iter__(self):
-            return self
+            self.__parent_keys = list(parentclass.keys())
 
         def __next__(self):
             if self.__counter < self.__parent_len:
                 key = self.__parent_keys[self.__counter]
-                value = self.__parentclass._dns_dict[key]
+                value = self.__parentclass[key]
                 self.__counter += 1
                 return (key, value)
 
